import numpy as np

# (2, 7)
# (7, 7)
# (7, 1)

# pydoc3 torch.nn.Linear
#
#  torch.nn.Linear(in_features: int, out_features: int, bias: bool = True, device=None, dtype=None) -> None
#  |  
#  |  Applies a linear transformation to the incoming data: :math:`y = xA^T + b`

# La función se llama con las entradas y luego las salidas,
# pero almacena la matriz de las salidas contra las entradas,
# y por esto se usa la transpuesta de esa matriz.

# (2, 7)
# (7, 2)
Capa1 = np.fromfile( '0', dtype=np.float32 )
Capa1 = Capa1.reshape( (-1,2) )

# (7, 7)
Capa2 = np.fromfile( '2', dtype=np.float32 )
Capa2 = Capa2.reshape( (-1,7) )

# (7, 1)
# (1, 7)
Capa3 = np.fromfile( '4', dtype=np.float32 )
Capa3 = Capa3.reshape( (-1,7) )

# Los sesgos
Sesgos1 = np.fromfile( '1', dtype=np.float32 )
Sesgos2 = np.fromfile( '3', dtype=np.float32 )
Sesgos3 = np.fromfile( '5', dtype=np.float32 )

# La función de activación a la salida
# logistic

# El modelo realiza:
# En la primera capa
#      u = suma entradas_i * pesos_1 + sesgo_i
#                5000 x 2    2 x 7
#                  5000 x 7            + 7
#      y = relu( u )
#
# En la segunda capa
#      suma pesos_i * entradas_i + sesgo_i
#      y = relu( u )
# 
# En la tercera capa
#      suma pesos_i * entradas_i + sesgo_i
#      y = 1/(1+exp(-u ))

# Suponemos que el "Entradas" es una matriz
# de tamaño n x características
def evalua( Entradas ) : 
	# print( "Capa1:", Capa1 )
	m, n = Entradas.shape

	if n != 2 :
		print( "La matriz de entrada debe tener dos columnas" )
		return 0

	# Realizamos los cálculos en la primera capa
	# Entrada n x 2
	# Capa1   2 x 7
	U1 = Entradas @ Capa1.T + Sesgos1
	U1_ = np.where( U1 < 0, 0, U1 )

	# U2 es de tamaño 7 x 1
	# Capa2 es de tamaño 7x7
	U2 = U1_ @ Capa2.T + Sesgos2
	U2_ = np.where( U2 < 0, 0, U2 )

	# U3 es la tercera capa
	U3 = U2_ @ Capa3.T + Sesgos3
	U3_ = 1.0/( 1.0+np.exp(-U3) )

	return U3_

